open Ast

let string_of_variable = function
		ArrayVar(s, i) -> "int[] " ^ s ^ "= new int[" ^ string_of_int i ^ "];\n" 
	| IntVar(s) -> "int " ^ s ^ ";\n"

let rec string_of_variablecall = function
	  IntVarCall(s) -> s
	| ArrayVarCall(s, e) -> s ^ "[" ^ string_of_expr e ^ "]"
and string_of_expr = function
    Literal(l) -> string_of_int l
 	| Var(s) -> string_of_variablecall s
  | Binop(e1, o, e2) ->
      string_of_expr e1 ^ " " ^
      (match o with
	Add -> "+" | Sub -> "-" | Mult -> "*" | Div -> "/" | Mod -> "%"
      | Equal -> "==" | Neq -> "!="
      | Less -> "<" | Leq -> "<=" | Greater -> ">" | Geq -> ">=") ^ " " ^
      string_of_expr e2
	| Uniop(o, e) -> "AMath." ^ (match o with
		Sqrt -> "sqrt("  
	| Sum -> "sum("  
	| Avg -> "avg(" 
	| Abs -> "abs("
	| Log -> "log(" ) ^ 
		string_of_expr e ^ ")"		
  | Assign(v, e) -> string_of_variablecall v ^ " = " ^ string_of_expr e
	| Power(e1, e2) -> "AMath.pow(" ^ string_of_expr e1 ^ ", " ^ string_of_expr e2 ^ ")"
  | Call(f, el) -> 		
		 "AAL." ^ f ^ "(" ^ String.concat ", " (List.map string_of_expr el) ^ ")"
  | Noexpr -> ""

let rec string_of_stmt = function
    Block(stmts) ->
      "{\n" ^ String.concat "" (List.map string_of_stmt stmts) ^ "}\n"
  | Expr(expr) -> string_of_expr expr ^ ";\n";
  | Return(expr) -> "return " ^ string_of_expr expr ^ ";\n";
  | If(e, s, Block([])) -> "if (" ^ string_of_expr e ^ ")\n" ^ string_of_stmt s
  | If(e, s1, s2) ->  "if (" ^ string_of_expr e ^ ")\n" ^
      string_of_stmt s1 ^ "else\n" ^ string_of_stmt s2
  | For(e1, e2, e3, s) ->
      "for (" ^ string_of_expr e1  ^ " ; " ^ string_of_expr e2 ^ " ; " ^
      string_of_expr e3  ^ ") " ^ string_of_stmt s
  | While(e, s) -> "while (" ^ string_of_expr e ^ ") " ^ string_of_stmt s
	| Print(e) -> "System.out.println(" ^ string_of_expr e ^ ");\n"

let string_of_fparam = function
    Int(id) -> "int " ^ id
  | Array(id) -> "int[] "^ id

let string_of_functiontype = function
		VoidRet -> "void"
	| IntRet -> "int"
	| ArrayRet -> "int[]" 	

let string_of_fdecl fdecl =
  "public static " ^ string_of_functiontype fdecl.ftype ^ " " ^
	fdecl.fname ^ "(" ^ String.concat ", " (List.map string_of_fparam fdecl.formals) ^ ")\n{\n" ^
  String.concat "" (List.map string_of_variable fdecl.locals) ^
  String.concat "" (List.map string_of_stmt fdecl.body) ^
  "}\n"

let string_of_program funcs =
	"public class AAL { \n" ^
  String.concat "\n" (List.map string_of_fdecl funcs) ^ "\n" ^
	"public static void main(String args[]){"^
	"AAL.execute();"^
	"}"^
	"} \n"
